require 'daruma/version'

require 'net/http'
require 'uri'

require 'kudan'

module Daruma
  class Client
    attr_reader :host, :port

    def initialize(base_url)
      uri = URI.parse(base_url)

      @host = uri.host
      @port = uri.port
    end

    class << self
      def define_http_method(method_name, http_class)
        define_method(method_name) do |url, params: nil, tries: 0, sleep_seconds: 0|
          req = http_class.new(url)
          req.set_form_data(params) unless params.nil?

          Kudan.retryable(tries: tries, sleep_seconds: sleep_seconds) do
            response = Net::HTTP.start(@host, @port) { |http| http.request(req) }
            response.value
            response
          end
        end
      end
    end

    define_http_method :get,    Net::HTTP::Get
    define_http_method :post,   Net::HTTP::Post
    define_http_method :put,    Net::HTTP::Put
    define_http_method :patch,  Net::HTTP::Patch
    define_http_method :delete, Net::HTTP::Delete
  end
end
